#!/usr/bin/python3
'''
OpenQP : Question Bank Management System
Authors  : Ajith Kumar B.P (bpajith@gmail.com)
           T K Muraleedharan (muraleedharantk@kannuruniv.ac.in)
License : GNU GPL version 3
Edited on : 11-May-2025

Selects a course. Downloads data from a selected course to an Excel file, created in a new folder.
Images are written to the same folder.
'''

import pandas as pd
import xlsxwriter
import subprocess, os, sys, pathlib, time, psycopg2, shutil
from psycopg2 import OperationalError
import numpy as np
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *

#from sys import platform
#from pathlib import Path

 
CC1 = 0		# coursedetails table
CC2 = 1
CC3 = 2
NAME = 3
SEM = 4
LANG = 5
QPTYPE = 6
NMODS = 7
DUR = 8
DEG = 9

class MainWindow(QMainWindow):   # Creating main window class
    editorCursor = None	
    count = 0
    codeFilter = ''
    courseList = []
    courseCode = None
    myhost = None
    mydb = None
    myport = None    
    conn = None
    cursor = None

    def __init__(self, *args, **kwargs):
        super(MainWindow, self).__init__(*args, **kwargs)
        
        palette = QPalette()
        palette.setColor(QPalette.ColorRole(10), QColor(150,175,150)) 
        self.setPalette(palette)	
        
        self.setGeometry(100, 100, 1100, 600)			
        fixedfont = QFontDatabase.systemFont(QFontDatabase.FixedFont)
        fixedfont.setPointSize(14)

        layout = QVBoxLayout()
        hbox1 = QHBoxLayout()
        layout.addLayout(hbox1)			
        hbox2 = QHBoxLayout()
        layout.addLayout(hbox2)			
        hbox3 = QHBoxLayout()
        layout.addLayout(hbox3)			
        hbox4 = QHBoxLayout()
        layout.addLayout(hbox4)			
        hbox5 = QHBoxLayout()
        layout.addLayout(hbox5)			
        hbox6 = QHBoxLayout()
        layout.addLayout(hbox6)			
        
        self.dbSelectButton = QPushButton('Select DB')
        hbox1.addWidget(self.dbSelectButton)
        self.dbSelectButton.clicked.connect(self.readConfig)	
        
        self.dbConnectButton = QPushButton('Connect to QB')
        s = 'Connects to the Database and bring the list of the available courses\nYou may choose any from the "Select Course" pull down menu'
        self.dbConnectButton.setToolTip(s)
        hbox1.addWidget(self.dbConnectButton)
        self.dbConnectButton.clicked.connect(self.updateCourseList)	

        hbox1.addWidget(QLabel('Filter->'))	
        self.ccLineEdit = QLineEdit('')		
        hbox1.addWidget(self.ccLineEdit)
        self.ccLineEdit.textChanged.connect(self.setCodeFilter)		
        
        self.countLabel = QLabel('   ')
        hbox1.addWidget(self.countLabel)
        
        self.courseMenu = QMenu()
        self.courseSelectButton = QPushButton('Select Course')
        s = 'Choose a Course'
        self.courseSelectButton.setToolTip(s)
        self.courseSelectButton.setMenu(self.courseMenu)
        hbox1.addWidget(self.courseSelectButton)
        hbox1.addStretch()	

        self.downloadButton = QPushButton('Download Course')
        hbox1.addWidget(self.downloadButton)
        self.downloadButton.clicked.connect(self.downloadCourse)	
            
        self.qeditor = QPlainTextEdit()
        self.qeditor.setFont(fixedfont)
        self.qeditor.setLineWrapMode(0)
        self.qeditor.setReadOnly(True)			
        hbox5.addWidget(self.qeditor)	
        
        self.msgLabel = QLabel('')
        hbox6.addWidget(self.msgLabel)			
        container = QWidget()
        container.setLayout(layout)
        self.setCentralWidget(container)  	# making it central widget
        self.setWindowTitle('Download Course to Excel file')
        self.show()
        
    def readConfig(self): # Get the database details from config.txt
        dialog = QFileDialog()
        path, _ = dialog.getOpenFileName(GUI, "Open Configuration file (config.txt)", "","Text Files(*.txt)")        
        if len(path) == 0:  # user cancelled selection
            return				
        try:
            f = open(path)		
            while 1:
                ss = f.readline().strip()
                if ss == '':
                     break
                if ss[0] == '#':		#comments
                     continue
                item = ss.split(':')
                if item[0] == 'name':
                    self.myname = item[1]
                elif item[0] == 'host':
                    self.myhost = item[1]
                elif item[0] == 'port':
                    self.myport = int(item[1])
                elif item[0] == 'dbname':
                    self.mydb = item[1]
        except:
            self.dialog_critical('Config file not found')
            return			
        
    def updateCourseList(self):
        self.courseList = []
        try:
            print('Connecting to %s Server ...'%(self.myname), end =' ')	
            sys.stdout.flush()	
            conn = psycopg2.connect(database= self.mydb, host=self.myhost, user='dummy', password="dum123", port=self.myport)
            cursor = conn.cursor()
            sql = "select code,code2,code3,name,semester, texlanguage,qpmodel, nmodules, duration, degree from coursedetails order by code;"
            cursor.execute(sql)
            res = cursor.fetchall()
            print ('Fetched data\n')
            conn.close()
            sys.stdout.flush()	
        except:
            self.dialog_critical('Connection to Database failed.')
            return		
        for k in res:
            self.courseList.append(k)
        self.setWindowTitle('%s Server. %d Courses available.'%(self.myname,len(self.courseList)))    
        self.displayCourses()
        
    def dispMessage(self,s):
        self.msgLabel.setText(s)

    def dialog_critical(self, s):
        dlg = QMessageBox(self)
        dlg.setText(s)
        dlg.setIcon(QMessageBox.Critical)
        dlg.show()
    
    def getPassword(self):
        text, ok = QInputDialog.getText(None, "QP Generation", "Password?", 
        QLineEdit.Password)
        if ok and text:
            return text		
            
    def closeEvent(self, event):
        if self.cursor != None:
            self.cursor.close()
            print('closed cursor')
        if self.conn != None:
            self.conn.close()
            print('closed connection')

    def setCodeFilter(self):
        self.codeFilter = self.ccLineEdit.text().upper()
        self.displayCourses()
    
    def displayCourses(self):
        #print('In displayCourses')		
        self.qeditor.setPlainText('')	
        self.editorCursor = self.qeditor.textCursor()        	
        txt = '%-15s %-15s %-15s %-6s %-50s\n'%('Course Code','Code2','Code3','QPtype','Course Name') 
        self.editorCursor.insertText(txt)     
        self.courseMenu.clear()
        self.count = 0
        for row in self.courseList:
            ss = row[CC1] + row[CC2] + row[CC3]
            if self.codeFilter != '' and self.codeFilter not in ss:  	
                continue		
            self.courseMenu.addAction(row[0], lambda item=row[0]: self.selectCourse(item))	
            self.count += 1      
            cc2 = row[CC2]
            cc3 = row[CC3] 
            s = '%-15s %-15s %-15s %-6s %-50s\n'%(row[CC1][8:],cc2,cc3,row[QPTYPE],row[NAME].strip())
            #s = 'hello\n'
            self.editorCursor.insertText(s)  
        #self.qeditor.setPlainText(txt)
        self.countLabel.setText('%d'%self.count)
        
    def getPassword(self, cname):
        dialog = QInputDialog(self)
        dialog.setFixedSize(700, 100)
        text, ok = dialog.getText(None, "Course %s"%cname, "Password?", 
        QLineEdit.Password)
        if ok and text:
            return text		
                    
    def selectCourse(self, code):  # select a Course from DB	
        for row in self.courseList:
            if row[CC1] == code:
                self.courseCode = code	
                self.displayCourses()		
        pw = self.getPassword(code)	
        if pw == None:
            GUI.dialog_critical('No password obtained for %s.'%code)	
            return
            
        try:
            self.conn = psycopg2.connect(host=self.myhost, database= self.mydb, port=self.myport, user=self.mydb.lower(), password=pw)
            self.cursor = self.conn.cursor()			
            self.cursor.execute("SELECT count(*) FROM %s"%self.courseCode)	
            res = self.cursor.fetchone()
        except OperationalError as e:
            self.dialog_critical('%s'%e)
            return					
        print(self.courseCode, res)		
                            
    def downloadCourse(self):
        if self.courseCode == None:
            self.dialog_critical('No course selected')
            return
        if os.path.exists(self.courseCode):
            shutil.rmtree(self.courseCode)	
        os.mkdir(self.courseCode)
        os.chdir(self.courseCode)
        df = pd.DataFrame(columns=['modnum','unitrel','qtype','mark','level','outcome','author','qtext','atext'])
        #Get the figures first
        try:
            sql = "SELECT figname,figure FROM %s where figname is not null"%self.courseCode
            self.cursor.execute(sql)	
            res = self.cursor.fetchall()
            for k in res: 
                fname = k[0]		
                figure = k[1]
                f = open(fname,'wb')
                f.write(figure)
                f.close()
        except OperationalError as e:
            print('Import figure Failed: %s'%e)
            sys.exit()
        
        sql = "SELECT modnum, unitnum, qtype, mark, level, outcome,author,qtext,atext FROM %s where figname is null order by modnum,qtype"%self.courseCode
        self.cursor.execute(sql)
        res = self.cursor.fetchall()
        NR = len(res)
        for k in range(NR):
            row = res[k]
            if row[4].lower == 'inactive':      # discard inactive ones
                continue
            #print(row)
            df.loc[k] =[row[0], row[1], row[2], row[3], row[4], row[5], row[6], row[7], row[8]]
            
        writer = pd.ExcelWriter(self.courseCode + '.xlsx', engine='xlsxwriter')
        df.to_excel(writer, sheet_name='Sheet1', index=False)
        workbook = writer.book
        
        wrap_format = writer.book.add_format({'text_wrap': True})	
        header2 = workbook.add_format({
        'align':    'left',
        'valign':   'top',
        'font_size': 12,
        'text_wrap': True
            })

        worksheet = writer.sheets['Sheet1']
        worksheet.set_column(0,0,7)
        worksheet.set_column(1,1,8)
        worksheet.set_column(2,2,5)
        worksheet.set_column(4,4,12)
        worksheet.set_column(6,6,20)
        worksheet.set_column('H:I', 60,header2)
        rows, columns = df.shape
        for k in range(rows):
            qt = df.iloc[k]['qtext']
            hq = (1 + len(qt)//60 + qt.count('\n')) * 15
            at = df.iloc[k]['atext']
            ha = (1 + len(at)//60 + qt.count('\n')) * 15
            h = hq if (hq>ha) else ha
            #print(k, hq, ha, h) 
            worksheet.set_row(k+1, h, header2)
        writer.save()	
        print(self.courseCode,rows)
        os.chdir('..')
        
app = QApplication(sys.argv)   # Main program
GUI = MainWindow()
app.exec_()		# main loop

