#!/usr/bin/python3
'''
OpenQP : Question Bank Management System
Authors  : Ajith Kumar B.P (bpajith@gmail.com)
           T K Muraleedharan (muraleedharantk@kannuruniv.ac.in)
License : GNU GPL version 3
Last edit : 10-May-2025
'''

from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
import subprocess, os, sys, pathlib, time, psycopg2, threading
from psycopg2 import OperationalError
import numpy as np
from sys import platform
from pathlib import Path
import qpmodule

 
CC1 = 0		# coursedetails table
CC2 = 1
CC3 = 2
NAME = 3
SEM = 4
LANG = 5
QPTYPE = 6
NMODS = 7
DUR = 8
DEG = 9

semesterFilters = ['All','First','Second','Third','Fourth','Fifth', 'Sixth', 'Seventh', 'Eighth']
    
if platform.startswith("win"):
    pdfviewer="start"
    shell = True
else:
    pdfviewer="evince"
    shell = False		


class MainWindow(QMainWindow):   # Creating main window class
    editorCursor = None	
    count = 0
    codeFilter = ''
    semFilter = semesterFilters[0]	
    courseList = []
    courseCode = None
    language = 'english'	# needed for generating LaTex
    myname = None	  # user selects one
    myhost = None
    mydb = None
    myport = None    
    conn = None
    cursor = None
    year = '---'
    month = '---'
    qpcode = '---'
    ccdisp = [1,0,0]  # Select the Course codes to be displayed    
    
    def __init__(self, *args, **kwargs):
        super(MainWindow, self).__init__(*args, **kwargs)
        
        palette = QPalette()
        palette.setColor(QPalette.ColorRole(10), QColor(150,175,150)) 
        self.setPalette(palette)	
        
        self.setGeometry(100, 100, 1100, 600)			
        fixedfont = QFontDatabase.systemFont(QFontDatabase.FixedFont)
        fixedfont.setPointSize(14)

        layout = QVBoxLayout()
        hbox1 = QHBoxLayout()
        layout.addLayout(hbox1)			
        hbox2 = QHBoxLayout()
        layout.addLayout(hbox2)			
        hbox3 = QHBoxLayout()
        layout.addLayout(hbox3)			
        hbox4 = QHBoxLayout()
        layout.addLayout(hbox4)			
        hbox5 = QHBoxLayout()
        layout.addLayout(hbox5)			
        hbox6 = QHBoxLayout()
        layout.addLayout(hbox6)			
        
        self.dbSelectButton = QPushButton('Select DB')
        hbox1.addWidget(self.dbSelectButton)
        self.dbSelectButton.clicked.connect(self.readConfig)	

        self.dbConnectButton = QPushButton('Connect to QB')
        s = 'Connects to the Database and bring the list of the available courses\nYou may choose any from the "Select Course" pull down menu'
        self.dbConnectButton.setToolTip(s)
        hbox1.addWidget(self.dbConnectButton)
        self.dbConnectButton.clicked.connect(self.updateCourseList)	

        hbox1.addWidget(QLabel('Filter->'))	
        self.ccLineEdit = QLineEdit('')		
        hbox1.addWidget(self.ccLineEdit)
        self.ccLineEdit.textChanged.connect(self.setCodeFilter)		
        
        self.countLabel = QLabel('   ')
        hbox1.addWidget(self.countLabel)
        
        self.courseMenu = QMenu()
        self.courseSelectButton = QPushButton('Select Course')
        s = 'Choose a Course'
        self.courseSelectButton.setToolTip(s)
        self.courseSelectButton.setMenu(self.courseMenu)
        hbox1.addWidget(self.courseSelectButton)
        hbox1.addStretch()	
                
        self.filterSemesterButton = QPushButton('Semester: All')
        hbox1.addWidget(self.filterSemesterButton)
        m = QMenu()
        for sub in semesterFilters:
            m.addAction(sub, lambda item=sub: self.setSemesterFilter(item))
        self.filterSemesterButton.setMenu(m)
                
        hbox3.addWidget(QLabel('Year'))
        self.yearLineEdit = QLineEdit('')
        self.yearLineEdit.setMaximumWidth(100)
        hbox3.addWidget(self.yearLineEdit)
        hbox3.addWidget(QLabel('Month'))
        self.monthLineEdit = QLineEdit('')
        self.monthLineEdit.setMaximumWidth(100)
        hbox3.addWidget(self.monthLineEdit)
         
        hbox3.addWidget(QLabel('QP Code'))	
        self.qpCodeLineEdit = QLineEdit('')
        self.qpCodeLineEdit.setMaximumWidth(100)
        hbox3.addWidget(self.qpCodeLineEdit)
        self.supCB = QCheckBox('Supplementary')
        hbox3.addWidget(self.supCB)        
        hbox3.addStretch()			

        self.ccLabel = QLabel('Select the CourseCodes to be printed->')
        hbox4.addWidget(self.ccLabel)	

        self.cc1CB = QCheckBox(15*'-')
        hbox4.addWidget(self.cc1CB)
        
        self.cc2CB = QCheckBox(15*'-')
        self.cc2CB.setVisible(False)
        hbox4.addWidget(self.cc2CB)
        
        self.cc3CB = QCheckBox(15*'-')
        self.cc3CB.setVisible(False)
        hbox4.addWidget(self.cc3CB)

        hbox4.addStretch()				
        self.qpButton = QPushButton('Generate QP')
        s = 'Initiates making a Question Paper\nby bringing questions from the database.\nmay take several seconds to finish. Press the Result Button if no PDF output'
        self.qpButton.setToolTip(s)		
        self.qpButton.clicked.connect(self.genQP)
        hbox4.addWidget(self.qpButton)
        self.resButton = QPushButton('Result')
        hbox4.addWidget(self.resButton)
        self.resButton.clicked.connect(self.showResult)		

        self.qeditor = QPlainTextEdit()
        self.qeditor.setFont(fixedfont)
        self.qeditor.setLineWrapMode(0)
        self.qeditor.setReadOnly(True)			
        hbox5.addWidget(self.qeditor)	
        
        self.msgLabel = QLabel('')
        hbox6.addWidget(self.msgLabel)			
        container = QWidget()
        container.setLayout(layout)
        self.setCentralWidget(container)  	# making it central widget
        self.setWindowTitle('Question paper Generator')
        self.show()
        
    def readConfig(self): # Get the database details from config.txt
        dialog = QFileDialog()
        path, _ = dialog.getOpenFileName(GUI, "Open Configuration file (config.txt)", "","Text Files(*.txt)")        
        if len(path) == 0:  # user cancelled selection
            return		
            
        try:
            f = open(path)
            while 1:
                ss = f.readline().strip()
                if ss == '':
                     break
                if ss[0] == '#':		#comments
                     continue
                item = ss.split(':')
                if item[0] == 'name':
                    self.myname = item[1]
                elif item[0] == 'host':
                    self.myhost = item[1]
                elif item[0] == 'port':
                    self.myport = int(item[1])
                elif item[0] == 'dbname':
                    self.mydb = item[1]
        except:
            self.dialog_critical('File "Config.txt" not found in the home directory')
        self.setWindowTitle('Using %s Server'%self.myname)
        print(self.myhost, self.mydb,self.myport)    

    def showResult(self):
        fn = 'output/%s.txt'%self.courseCode
        s = open(fn).read()
        self.qeditor.setPlainText(s)
        
    def dispMessage(self,s):
        self.msgLabel.setText(s)

    def dialog_critical(self, s):
        dlg = QMessageBox(self)
        dlg.setText(s)
        dlg.setIcon(QMessageBox.Critical)
        dlg.show()
    
    def getPassword(self):
        text, ok = QInputDialog.getText(None, "QP Generation", "Database Read Password?", 
        QLineEdit.Password)
        if ok and text:
            return text		
            
    def closeEvent(self, event):
        if self.cursor != None:
            self.cursor.close()
            print('closed cursor')
        if self.conn != None:
            self.conn.close()
            print('closed connection')

    def setSemesterFilter(self,s):
        self.semFilter = s
        self.filterSemesterButton.setText('Semester:'+s)
        self.displayCourses()

    def setCodeFilter(self):
        self.codeFilter = self.ccLineEdit.text().upper()
        self.displayCourses()
    
    def displayCourses(self):
        #print('In displayCourses')		
        self.qeditor.setPlainText('')	
        self.editorCursor = self.qeditor.textCursor()        	
        txt = '%-15s %-15s %-15s %-6s %-50s\n'%('Course Code','Code2','Code3','QPtype','Course Name') 
        self.editorCursor.insertText(txt)     
        self.courseMenu.clear()
        self.count = 0
        for row in self.courseList:
            ss = row[CC1] + row[CC2] + row[CC3]
            if self.codeFilter != '' and self.codeFilter not in ss:  	
                continue		
            if self.semFilter != 'All' and self.semFilter != row[SEM]:  	
                continue
            self.courseMenu.addAction(row[0], lambda item=row[0]: self.selectCourse(item))	
            self.count += 1      
            cc2 = row[CC2]
            cc3 = row[CC3] 
            s = '%-15s %-15s %-15s %-6s %-50s\n'%(row[CC1][8:],cc2,cc3,row[QPTYPE],row[NAME].strip())
            #s = 'hello\n'
            self.editorCursor.insertText(s)  
        #self.qeditor.setPlainText(txt)
        self.countLabel.setText('%d'%self.count)
        
    def selectCourse(self, code):  # select a Course from DB	
        for row in self.courseList:
            if row[CC1] == code:
                self.courseCode = code	
                #self.ccLineEdit.setText(code[8:])
                self.cc1CB.setText(code[8:])
                self.cc1CB.setChecked(True)				               
                self.cc2CB.setText(row[CC2])
                self.cc3CB.setText(row[CC3])
                if row[CC2] != '':
                    self.cc2CB.setVisible(True)
                else:
                    self.cc2CB.setVisible(False)
                                        
                if row[CC3] != '':
                    self.cc3CB.setVisible(True)
                else:
                    self.cc3CB.setVisible(False)
                self.displayCourses()				
                            
    def updateCourseList(self):	        
        if self.myname == None:
            self.dialog_critical('Database not Selected.')	
            return
        pw = self.getPassword()	
        if pw == None:
            self.dialog_critical('Incorrect password.')	
            return
        self.passwd = pw
        self.courseList = []
        try:
            print('Connecting to %s Server...'%(self.myname), end =' ')	
            sys.stdout.flush()	
            conn = psycopg2.connect(database= self.mydb, host=self.myhost, user='dummy', password="dum123", port=self.myport)
            cursor = conn.cursor()
            sql = "select code,code2,code3,name,semester, texlanguage,qpmodel, nmodules, duration, degree from coursedetails order by code;"
            cursor.execute(sql)
            res = cursor.fetchall()
            print ('Fetched data\n')
            conn.close()
            sys.stdout.flush()	
            self.setWindowTitle('Connected to %s Server'%self.myname)
        except:
            self.dialog_critical('Connection to Database failed.')
            return		
        for k in res:
            self.courseList.append(k)
        self.dispMessage('Found %d Courses.'%len(self.courseList))
        self.displayCourses()
        
    def genQP(self):
        if self.courseCode == None:
            self.dialog_critical('Select a Course.')
            return	
            
        self.ccdisp = ''
        if self.cc1CB.isChecked():
            self.ccdisp += self.cc1CB.text()
        if self.cc2CB.isVisible() and self.cc2CB.isChecked():
            self.ccdisp += '/'+self.cc2CB.text()
        if self.cc3CB.isVisible() and self.cc3CB.isChecked():
            self.ccdisp += '/'+self.cc3CB.text()
        if self.ccdisp == '':
            self.dialog_critical('At least one Course Code should be checked')	
            return
        if self.ccdisp[0] == '/':
            self.ccdisp = self.ccdisp[1:]
                                
        self.year = self.yearLineEdit.text()
        self.month = self.monthLineEdit.text()
        #self.header = self.headerLineEdit.text()
        self.qpcode = self.qpCodeLineEdit.text()

        for x in pathlib.Path(".").glob("%s.*"%self.courseCode) : x.unlink()
        
        if not os.path.exists('output'):
            os.mkdir('output')

        os.chdir('output')        
        res = qpmodule.callQP(self.myhost, self.myport, self.mydb, 'reader', self.passwd, self.courseCode, self.year, self.month, self.qpcode,self.ccdisp,self.supCB.isChecked())
        
        if res != True:
            self.dialog_critical('Failed. Press the Result Button to view the log file')
            os.chdir('..')
            return
        fn = self.courseCode	# Tex input file
        # Running xelatex twice to get the page number correct.
        res = subprocess.run(["xelatex", '-halt-on-error', fn+'.tex'],capture_output=True,text=True)
        res = subprocess.run(["xelatex", '-halt-on-error', fn+'.tex'],capture_output=True,text=True)
        if res.returncode != 0:
            self.dialog_critical('Output from "xelatex" (last 100 characters):\n%s'%res.stdout[-100:])
        else:
            self.dispMessage('Generated %s.pdf\n'%fn)
        res = subprocess.run(["xelatex", '-halt-on-error', fn+'_ANS.tex'],capture_output=True,text=True)
        res = subprocess.run(["xelatex", '-halt-on-error', fn+'_ANS.tex'],capture_output=True,text=True)
        if res.returncode != 0:
            self.dialog_critical('Output from "xelatex" (last 100 characters):\n%s'%res.stdout[-100:])
        else:
            self.dispMessage('Generated %s_ANS.pdf\n'%fn)
        if os.path.exists(self.courseCode + '.pdf') == True:		
            subprocess.Popen(["evince", self.courseCode + '.pdf'])	
        else:
            print('making PDF failed. Examine "log.txt"')
        for x in pathlib.Path(".").glob("*.log") : x.unlink()
        for x in pathlib.Path(".").glob("*.aux") : x.unlink()
        os.chdir('..')


app = QApplication(sys.argv)   # Main program
GUI = MainWindow()
app.exec_()		# main loop

