#!/usr/bin/python3
'''
OpenQP : Question Bank Management System
Authors  : Ajith Kumar B.P (bpajith@gmail.com)
           T K Muraleedharan (muraleedharantk@kannuruniv.ac.in)
License : GNU GPL version 3
Last edit : 15-May-2025

Uploads data from Excel file to the Database, images and the Excel file must in the same folder. 
Old contents are deleted from the selected course.
The question numbers are generated by the code. 
'''

import psycopg2,sys, csv,os, subprocess, time
import pandas as pd
import numpy as np
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtCore import QThread, QObject, pyqtSignal as Signal, pyqtSlot as Slot

MAXMOD = 8		# KU has 8, others upto 5
        
languages = ['english','french', 'hindi', 'sanskrit','malayalam','tamil','kannada','urdu','arabic']

if sys.platform.startswith("win"):	# MS Windows
    pdfviewer="start"
    shell = True
else:
    pdfviewer="evince"
    shell = False

helpMsg = '''How to use the Question bank Uploader:

Step 1. Select the language, by default it is english. Must be done before opning the file.

Step 2. Open the .xlsx file. The contents will be displayed line by line. Stops if any error is found and the corresponding line will be diplayed. "File Loading Error" means your .xlsx file has a wrong format, like:
  a. The number of columns is not Nine. Delete several coulumns after 'atext' and try again.
  b. The sheet name is not "Sheet1". It is shown on left bottom of the spreadsheet. Rename it by right clicking on it.

Step 3. View Question in PDF
Generates a .tex file from the questions and calls the external program 'xelatex' to convert it into PDF format. Included figures must be in the same folder where the .xlsx file is kept.
On successful conversion, a PDF listing is displayed. Otherwise error message from 'xelatex' containing part of the offending text is displayed. Mostly it would be a &, % or $ in some question. Go to the .xlsx file, replace them like \& \% \$ and press "Open XLSX file" again .

Step 4: Open the course located on the remote Question Bank server. The password should be provided.

Step 5: Upload the Questions. If this fails with the message "Course is not Empty", press the "Clear Remote Course" button to clear it. Depending on your network speed, uploading may take several minutes.

Step 7: If you like, display the contents of the remote course. You can sort it module wise by enabling the checkbox. This step is only for verification.

The LaTex Testing Mode is NOT a part of the uploading process. It is provided to help preparing the questions. Before entering the questions into the spreadsheet one can check them for LaTex compatibility by generating a PDF version. Entering this mode disables the upload related buttons but doesn't affect the uploading process.
'''

class MainWindow(QMainWindow):   # Creating main window class   
    work_requested = Signal(int)	
    docdir = None
    columns = ['modnum', 'unitnum','qtype', 'mark','level','outcome','author','qtext','atext']
    datalist = []
    figlist = []
    tex = ''
    language = 'english'
    course = None
    editorCursor = None
    texMode = False
    myname = None	  # user selects one
    myhost = None
    mydb = None
    myport = None
    conn = None
    cursor = None
                      
    def openFile(self):
        dialog = QFileDialog()
        path, _ = dialog.getOpenFileName(GUI, "Open file", "","XLS Files(*.xlsx *.XLSX)")
        if path == '':
            return
        self.qeditor.setLineWrapMode(0)            
        self.qeditor.setPlainText('') 
        self.editorCursor = self.qeditor.textCursor()
        
        if 1:    
            df = pd.read_excel(path, sheet_name=0, index_col=None, usecols = self.columns)
            df.fillna('', inplace = True)
        else: 
            self.editorCursor.insertText('File loading Error')
            return	
        
        nrows = len(df)
        print(nrows)
        line = 0	
        self.datalist = []
        
        self.tex,tag,langbeg,langend = self.texHeader()	
        self.tex += '\\begin{document}\n'
        self.tex += '\\begin{enumerate}\n'
        
        for col in ['qtype', 'level', 'outcome', 'author', 'qtext', 'atext']:
            df[col] = df[col].str.strip()
        df['level'] = df['level'].str.capitalize()
        
        for j in range(nrows):  # validate & copy dataframe rows to the list  
            if df.loc[j].modnum == '$$$':  # out end of data flag
                break         
            line+=1
            try:
                modnum = int(df.loc[j].modnum)
                unitnum = df.loc[j].unitnum
                qtype = df.loc[j].qtype
                mark = int(df.loc[j].mark)
                level = df.loc[j].level
                outcome = df.loc[j].outcome
                author = df.loc[j].author
                question = df.loc[j].qtext
                answer = df.loc[j].atext
            except Exception as e:
                self.editorCursor.insertText(f'Parse Error in Row {line}\n')
                self.editorCursor.insertText(f'{line}: {df.loc[j]}')
                return
            s = f'{modnum},{unitnum},{qtype},{mark},{level},{outcome},{author},{question},{answer}\n'
            s = '%3d:%s %s %s %-2d% -10s %-4s %-11s %s | %s\n'%(line,modnum,unitnum,qtype,mark,level,outcome,author,question,answer)

            #Validation..
            if modnum not in range(1,9):
                self.editorCursor.insertText(f'Error in Row {line}. Invalid Module Number\n')
                self.editorCursor.insertText(f'{line}: {s}')
                self.qeditor.verticalScrollBar().setValue(self.qeditor.verticalScrollBar().maximum());
                return
            if qtype not in ['A', 'B', 'C', 'D']:
                self.editorCursor.insertText(f'Error in Row {line}. Invalid Question Type\n')
                self.editorCursor.insertText(f'{line}: {s}')
                self.qeditor.verticalScrollBar().setValue(self.qeditor.verticalScrollBar().maximum());
                return
            if mark not in range(1,16):
                self.editorCursor.insertText(f'Error in Row {line}. Invalid Mark\n')
                self.editorCursor.insertText(f'{line}: {s}')
                self.qeditor.verticalScrollBar().setValue(self.qeditor.verticalScrollBar().maximum());
                return
            if level == 'Analyse':
                level = 'Analyze'                
            if level not in ['Remember', 'Understand', 'Apply', 'Analyze','Evaluate', 'Create', 'inactive']:
                self.editorCursor.insertText(f'Error in Row {line}. Invalid Level\n')
                self.editorCursor.insertText(f'{line}: {s}')
                self.qeditor.verticalScrollBar().setValue(self.qeditor.verticalScrollBar().maximum());
                return

            if outcome not in ['CO1', 'CO2', 'CO3', 'CO4', 'CO5', 'CO6', 'CO7']:
                self.editorCursor.insertText(f'Error in Row {line}. Invalid Outcome\n')
                self.editorCursor.insertText(f'{line}: {s}')
                self.qeditor.verticalScrollBar().setValue(self.qeditor.verticalScrollBar().maximum());
                return
            if len(author) < 3:
                self.editorCursor.insertText(f'Error in Row {line}. Author name too short\n')
                self.editorCursor.insertText(f'{line}: {s}')
                self.qeditor.verticalScrollBar().setValue(self.qeditor.verticalScrollBar().maximum());
                return
            if len(question) < 5:
                self.editorCursor.insertText(f'Error in Row {line}. Question too short\n')
                self.editorCursor.insertText(f'{line}: {s}')
                self.qeditor.verticalScrollBar().setValue(self.qeditor.verticalScrollBar().maximum());
                return
            self.editorCursor.insertText(s)
            self.tex += '\\item M%d%2s(%s,%9s,%4s) \\\\ \n'%(modnum,qtype,unitnum,level,outcome)
            self.tex += langbeg
            self.tex += '%s\n\n%s\n'%(question,answer)
            self.tex += langend
            self.datalist.append([modnum, unitnum, qtype, mark, level, outcome, author, question, answer])

        self.figlist = self.makeFigList(self.tex)
        self.tex += '\\end{enumerate}\n\\end{document}\n'		
        s = 'Read %d rows from %s'%(line,os.path.basename(path))	
        self.editorCursor.insertText('\n'+s)
       
        self.course = os.path.basename(path).split('.')[0]
        self.docdir = os.path.dirname(path)
        os.chdir(self.docdir)
        s = '\nCourse Code : %s\n'%self.course
        self.editorCursor.insertText(s)
        #print(self.course)
        self.qeditor.verticalScrollBar().setValue(self.qeditor.verticalScrollBar().maximum());
        self.setWindowTitle('Opened File %s'%os.path.basename(path))        

    def selectLanguage(self,s):
        self.language = s
        self.langButton.setText(s)
    
    def texHeader(self):
        tex =  '\\documentclass[12pt, a4paper]{article}\n'
        tex += '\\usepackage{graphicx}\n'
        tex += '\\usepackage{amsmath}\n \\usepackage{amssymb}\n'
        tex += '\\usepackage[scr=rsfs]{mathalpha}\n'
        tex += '\\usepackage[, headsep=0pt, left=0.8in, top=0.7in, bottom=0.7in]{geometry}\n'
        tex += '\\usepackage{enumitem}\n'
        tex += '\\usepackage{multicol}\n'
        tex += '\\usepackage{fancyhdr}\n'
        tex += '\\usepackage{fontspec}\n'
        tex += '\\usepackage{tfrupee}\n'
        langbeg = ''
        langend = ''
        tag = 'english'
        if self.language.lower() == 'malayalam':
            tag = 'malayalam'
            langbeg = '\\begin{malayalam}'
            langend = '\\end{malayalam}'
            tex += '\\usepackage{polyglossia}\n'
            tex += '\\setmainlanguage{english}'
            tex += '\\setotherlanguage{malayalam}\n'
            tex += '\\newfontfamily\\malayalamfont[Script=Malayalam]{Rachana}'
        elif self.language.lower() == 'tamil':
            tag = 'tamil'	
            langbeg = '\\begin{tamil}'
            langend = '\\end{tamil}'
            tex += '\\usepackage{polyglossia}\n'
            tex += '\\setmainlanguage{english}'
            tex += '\\setotherlanguage{tamil}\n'
            tex += '\\newfontfamily\\tamilfont[Script=Tamil]{Lohit-Tamil.ttf}\n'
        elif self.language.lower() == 'kannada':
            tag = 'kannada'	
            langbeg = '\\begin{kannada}'
            langend = '\\end{kannada}'
            tex += '\\usepackage{polyglossia}\n'
            tex += '\\setmainlanguage{english}'
            tex += '\\setotherlanguage{kannada}\n'
            tex += '\\newfontfamily\\kannadafont[Script=Kannada]{Lohit-Kannada.ttf}\n'
        elif self.language.lower() == 'hindi':
            tag = 'hindi'	
            langbeg = '\\begin{hindi}'
            langend = '\\end{hindi}'
            tex += '\\usepackage{polyglossia}\n'
            tex += '\\setmainlanguage{english}'
            tex += '\\setotherlanguage{hindi}\n'
            tex += '\\setmainfont[Script=Devanagari]{Lohit Devanagari}\n'
        elif self.language.lower() == 'sanskrit':
            tag = 'hindi'	
            langbeg = '\\begin{hindi}'
            langend = '\\end{hindi}'
            tex += '\\usepackage{polyglossia}\n'
            tex += '\\setmainlanguage{english}'
            tex += '\\setotherlanguage{hindi}\n'
            tex += '\\setmainfont[Script=Devanagari]{Lohit Devanagari}\n'
        elif self.language.lower() == 'arabic':
            tag = 'Arabic'
            langbeg = '\\begin{Arabic}'
            langend = '\\end{Arabic}'
            tex += '\\usepackage{polyglossia}\n'
            tex += '\\setmainlanguage{english}'
            tex += '\\setotherlanguage{arabic}\n'
            tex += '\\setmainfont[Script=Arabic,Scale=1.1]{Scheherazade}\n'	
        elif self.language.lower() == 'urdu':
            tag = 'urdu'
            langbeg = '\\begin{urdu}'
            langend = '\\end{urdu}'
            tex += '\\usepackage{polyglossia}\n'
            tex += '\\setmainlanguage{english}'
            tex += '\\setotherlanguage{urdu}\n'
            tex += '\\newfontfamily\\urdufont[Script=Arabic,Scale=1.0] {NafeesNastaleeq.ttf}\n'
        elif self.language.lower() == 'french':
            tag = 'english'
            langbeg = ''
            langend = ''	
        return tex,tag,langbeg,langend

    def makeFigList(self,tex):
        fragments = tex.split('\\includegraphics')	#search included figures
        self.figlist = []
        nfig = len(fragments)
        for k in range(1,nfig):
            s1 = fragments[k]
            s2 = s1.split('{')[1]
            print(k, ' s1 = ',s1, 's2 = ',s2)
            self.figlist.append(s2.split('}')[0])
        print(self.figlist)


    def verifyEach(self):
        self.msg.setText("Compiling Questions one by one. May it a lot of time")
        header,tag,langbeg,langend = self.texHeader()	
        header += '\\begin{document}\n'
        line = 1
        for row in self.datalist:
            tex = header + row[7] + row[8] + '\\end{document}\n'
            f = open('qqq.tex', 'w', encoding="utf-16")
            f.write(tex)
            f.close()
            line += 1
            print(line)
            # Run xelatex process  
            res = subprocess.run(["xelatex", '-halt-on-error', 'qqq.tex'],capture_output=True,text=True)           
            if res.returncode != 0:
                self.dialog_critical('Error: Excel row %d\n%s\n%s\n'%(line,row[7],row[8]))
                return
        self.dialog_critical('Verified all Questions.')
        
                    
    def processTex(self,tex, fn):
        self.msg.setText("Compiling code! Opening PDF soon...")
        try:
            with open(fn + '.tex', 'w', encoding="utf-16") as f:f.write(tex)
            # Remove auxiliary files if they exist
            if os.path.exists(fn + '.aux'):os.unlink(fn + '.aux')
            if os.path.exists(fn + '.pdf'):os.unlink(fn + '.pdf')

            # Run xelatex process  
            res = subprocess.run(["xelatex", '-halt-on-error', fn + '.tex'],      capture_output=True, text=True)
        
            if res.returncode != 0:
                # Emit error message if xelatex failed
                self.dialog_critical(f'Error from "xelatex":\n{res.stdout[-220:]}')
                return
            else:
                # Emit success message and open the PDF
                success_message = f'Generated {fn}.pdf'
                subprocess.Popen([pdfviewer, fn + '.pdf'], shell = shell)
        except:
            self.dialog_critical('Calling Latex Compiler Failed.')
            return		    
                                    
    def viewCourse(self):
        if self.course == None:
            self.dialog_critical('No XLSX file opened.')
            return		        
        self.processTex(self.tex, self.course)
        
    def dialog_critical(self, message):
        # Show a dialog with the critical error message (replace with your method)
        self.msg.setText("Compile crash : %s"%message)
        print(f"Critical error: {message}")
        dialog = QDialog()
        dialog.setWindowTitle("Error")
        dialog.exec_()
                
    def openCourse(self):	
        if self.course == None:
            self.dialog_critical('Open the XLSX file first.')
            return		
        if self.myname == None:
            self.dialog_critical('Select a Database.')
            return		
        try:
            self.conn = psycopg2.connect(database=self.mydb, host=self.myhost, user='dummy', password='dum123', port = self.myport)
            self.cursor = self.conn.cursor()
            sql = "select code from coursedetails where code = '%s';"%self.course
            self.cursor.execute(sql)
            res = self.cursor.fetchall()
            #print(res)
            if res == []:
                self.dialog_critical('%s not found.\nCheck your Filename.\nExample coursecode : Y2024AC_TES2CJ101\nCorresponding filename : Y2024AC_TES2CJ101.xlsx'%self.course)
                return
            self.cursor.close()
            self.conn.close()
            self.cursor = None
            self.conn = None            
        except:
            self.dialog_critical('Connecting to the Server Failed.')
            return
            
        pw = self.getPassword()
        myuser = self.course.lower()
        try:
            self.conn = psycopg2.connect(database=self.mydb, host=self.myhost, user=myuser, password=pw, port = self.myport)
            self.cursor = self.conn.cursor()
        except:
            self.dialog_critical('Connecting to %s Failed. Check password.'%self.course)
            return
        s = 'Connected to the Course %s\n'%self.course
        self.editorCursor.insertText(s)		
        self.setWindowTitle('Connected to Course %s'%self.course)        

    def clearCourse(self):
        if self.cursor == None:
            self.dialog_critical('Not connected to any Course yet')
            return
        reply = QMessageBox.question(GUI, 'Delete Course', 'All Data in this Course will Deleted. Proceed with Delete ?',QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
        if reply == QMessageBox.No:
            return
        sql = "delete from %s;"%(self.course)
        self.cursor.execute(sql)
        self.conn.commit()
        s='Deleted all records from %s\n'%self.course
        self.editorCursor.insertText(s)	
        
    def displayCourse(self):
        if self.cursor == None:
            self.dialog_critical('Not connected to any Course yet')
            return
        #Save all figures
        try:
            sql = "SELECT figname,figure FROM %s where figname is not null"%(self.course)
            self.cursor.execute(sql)	
            res = self.cursor.fetchall()	# will take only one.
            for k in res: 
                fname = k[0]		
                figure = k[1]
                f = open(fname,'wb')
                f.write(figure)
                f.close()
        except OperationalError as e:
            GUI.dialog_critical('Import figure: %s'%e)
            return            
        # Get all records, excluding figures. Only sorted option 
        sql = "select modnum, unitnum, qtype,qnum, mark, level, outcome,author,qtext,atext from %s where figname is null order by modnum,qtype,qnum;"%(self.course)		
        self.cursor.execute(sql)
        res = self.cursor.fetchall()
        self.qeditor.setPlainText('---------- Data Fetched from %s ---------\n'%self.course)
        self.editorCursor.insertText('')
        self.qeditor.verticalScrollBar().setValue(self.qeditor.verticalScrollBar().maximum());

        tex,tag,langbeg,langend = self.texHeader()	
        tex += '\\begin{document}\n'
        tex += '%s\n'%self.course.replace('_','\\_')
        tex += '\\begin{enumerate}\n'
        tex += langbeg
        for k in res:
            modnum = k[0]
            unitnum = k[1]
            qtype = k[2]
            qnum = k[3]
            mark = k[4]
            level = k[5]
            outcome = k[6]
            author = k[7]
            qtext = k[8]
            atext = k[9]          
            tex += '\\item M%s%2s %s (%s %9s %4s) \\\\ \n'%(modnum,qtype,qnum,unitnum,level,outcome)
            tex += '%s\n\n%s\n'%(qtext,atext)    
        tex += langend
        tex += '\\end{enumerate}\n\\end{document}\n'	            
        self.processTex(tex, 'aaa')

    def putFigures(self,tex,modnum):
        fragments = tex.split('\\includegraphics')	#search included figures
        figlist = []
        nfig = len(fragments)
        for k in range(1,nfig):
            s1 = fragments[k]
            s2 = s1.split('{')[1]
            figname = s2.split('}')[0]
            if os.path.exists(figname) == False:
                GUI.dialog_critical('File %s NOT found.'%figname)
                return				
            try:
                drawing = open(figname, 'rb').read()
                img = psycopg2.Binary(drawing)
                sql = "insert into %s(modnum,figname, figure) values (%d,'%s', %s);"%(self.course, modnum,figname, img)
                self.cursor.execute(sql)
            except OperationalError as e:
                self.cursor.execute('rollback;')			
                GUI.dialog_critical('%s'%e)
                return			
        return True		

    def upload(self,worker):
        print('Upload')
        if self.cursor == None:
            return
        print('Upload 2')
        # initialize question number storage        
        qna = {'A':[0]*MAXMOD, 'B':[0]*MAXMOD, 'C':[0]*MAXMOD, 'D':[0]*MAXMOD }        
        #check whether the course is empty 
        sql = "select count(*) from %s;"%(self.course)
        self.cursor.execute(sql)
        res = self.cursor.fetchone()
        if res[0] != 0:
            self.dialog_critical('%s is NOT empty'%self.course)
            return
        
        nrec = len(self.datalist)
        count = 0
        for row in self.datalist:
            modnum = row[0]
            qtype = row[2]
            print(count+1,modnum,qtype)
            qna[qtype][modnum-1] += 1	# generate Question number
            qnum = qna[qtype][modnum-1]
            sql = "insert into %s (modnum,unitnum,qtype,mark,level, outcome,author, qtext, atext, qnum) values (%d, '%s', '%s', %d, '%s', '%s', '%s', '%s','%s',%d);"%(self.course,row[0],row[1],row[2],row[3],row[4],row[5],row[6].replace("'","''"),row[7].replace("'","''"),row[8].replace("'","''"),qnum)
            #print(sql)
            self.cursor.execute(sql)
            if self.putFigures(row[7]+row[8],modnum) != True:
                return
            count += 1
            worker.progress.emit('Uploaded %d of %d'%(count,nrec))
        self.conn.commit()			
        s = 'Uploaded %d records\n'%count
        #self.editorCursor.insertText(s)		
        
    def setMode(self):
        if self.latexCB.isChecked():   
            self.qeditor.setPlainText('')
            self.qeditor.setLineWrapMode(1)
            self.qeditor.setReadOnly(False)
            self.texMode = True
            self.msg.setText('Editor Mode: Enter LaTex to be tested')
            self.fileButton.setEnabled(False)
            self.viewButton.setEnabled(False)
            self.openButton.setEnabled(False)
            self.uploadButton.setEnabled(False)
            self.clearButton.setEnabled(False)
            self.dispButton.setEnabled(False)
            self.latexButton.setEnabled(True)
        else:
            self.fileButton.setEnabled(True)
            self.viewButton.setEnabled(True)
            self.openButton.setEnabled(True)
            self.uploadButton.setEnabled(True)
            self.clearButton.setEnabled(True)
            self.dispButton.setEnabled(True)
            self.latexButton.setEnabled(False)
            self.qeditor.setPlainText('')		
            self.qeditor.setLineWrapMode(0)
            self.qeditor.setReadOnly(True)
            self.texMode = False
            self.msg.setText('Exiting Editor Mode')
            
    def tex2PDF(self):
        sss = self.qeditor.toPlainText()
        if len(sss) < 1:
            self.dialog_critical('Text area is Empty')           
            return
        tex,tag,langbeg,langend = self.texHeader()	
        tex += '\\begin{document}\n'
        tex += langbeg
        tex +=  self.qeditor.toPlainText()       
        tex += langend
        tex += '\\end{document}\n'
        self.processTex(tex,'qqq')
        
    def dialog_critical(self, s):
        dlg = QMessageBox(self)
        dlg.setText(s)
        dlg.setIcon(QMessageBox.Critical)
        dlg.show()
        
    def getPassword(self):
        text, ok = QInputDialog.getText(None, "Database", "Password?", 
        QLineEdit.Password)
        if ok and text:
            return text

    def readConfig(self): # Get the database details from config.txt
        dialog = QFileDialog()
        path, _ = dialog.getOpenFileName(GUI, "Open Configuration file (config.txt)", "","Text Files(*.txt)")        
        if len(path) == 0:  # user cancelled selection
            return		
		
        if 1:
            f = open(path)
            while 1:
                ss = f.readline().strip()
                if ss == '':
                     break
                if ss[0] == '#':		#comments
                     continue
                item = ss.split(':')
                if item[0] == 'name':
                    self.myname = item[1]
                elif item[0] == 'host':
                    self.myhost = item[1]
                elif item[0] == 'port':
                    self.myport = int(item[1])
                elif item[0] == 'dbname':
                    self.mydb = item[1]
        else:
            self.dialog_critical('Config file not found')
            return			
        self.setWindowTitle('Using %s Server.'%self.myname)
                                
    def __init__(self, *args, **kwargs):
        super(MainWindow, self).__init__(*args, **kwargs)
        palette = QPalette()
        palette.setColor(QPalette.ColorRole(10), QColor(150,175,150)) 
        self.setPalette(palette)	
        
        self.setGeometry(100, 100, 1100, 800)			
        fixedfont = QFontDatabase.systemFont(QFontDatabase.FixedFont)
        fixedfont.setPointSize(14)
        
        self.qeditor = QPlainTextEdit(helpMsg)
        self.qeditor.setFont(fixedfont)
        self.qeditor.setLineWrapMode(1)
        self.qeditor.setReadOnly(True)
        self.editorCursor = self.qeditor.textCursor()
        
        layout = QVBoxLayout()
        hbox1 = QHBoxLayout()
        layout.addLayout(hbox1)
        layout.addWidget(self.qeditor)

        self.langButton = QPushButton('english')
        self.langButton.setToolTip('Select the language before viewing PDF.' )
        hbox1.addWidget(self.langButton)
        m = QMenu()
        for s in languages:
            m.addAction(s, lambda item=s: self.selectLanguage(item))
        self.langButton.setMenu(m)
        
        self.fileButton = QPushButton('Open XLSX File')
        self.fileButton.setToolTip('The filename must be the coursecode, \
with a .xlsx extension.\nFor example Y2024AC_PHY2CJ101.xlsx' )         
        hbox1.addWidget(self.fileButton)
        self.fileButton.clicked.connect(self.openFile)	
        self.viewButton = QPushButton('View PDF')
        self.viewButton.setToolTip('Generates a single PDF file.\n')
        hbox1.addWidget(self.viewButton)
        self.viewButton.clicked.connect(self.viewCourse)
        self.verifyButton = QPushButton('Error Check')
        self.verifyButton.setToolTip('Checks the questions one by one for Latex errors. Displays the question with row number.\n')
        hbox1.addWidget(self.verifyButton)
        self.verifyButton.clicked.connect(self.verifyEach) 
        
            
        hbox1.addWidget(QLabel('|'))
        self.dbSelectButton = QPushButton('Select Database')
        hbox1.addWidget(self.dbSelectButton)
        self.dbSelectButton.clicked.connect(self.readConfig)	
        
        self.openButton = QPushButton('Open Remote Course')
        self.openButton.setToolTip('Connects to the Remote Database.\n\
The course code is the filename (without the .xlsx).\n\
You need to provide the password' )         
        self.openButton.setShortcut("Ctrl+R")
        hbox1.addWidget(self.openButton)
        self.openButton.clicked.connect(self.openCourse)	
        self.uploadButton = QPushButton('Upload Data')
        hbox1.addWidget(self.uploadButton)
        
        self.clearButton = QPushButton('Clear Remote Course')
        hbox1.addWidget(self.clearButton)
        self.clearButton.clicked.connect(self.clearCourse)	
        self.dispButton = QPushButton('Display Remote Course')
        hbox1.addWidget(self.dispButton)
        self.dispButton.clicked.connect(self.displayCourse)
        hbox1.addWidget(QLabel('|'))
        
        self.latexCB = QCheckBox('LaTex Testing Mode')
        hbox1.addWidget(self.latexCB)
        s = 'This mode is useful for testing the questions for LaTex correctness\
 before you enter them in to the Spreadsheet.\n\
         Press the button "LaTex to PDF" after entering your text.'
        self.latexCB.setToolTip(s)        
        self.latexCB.stateChanged.connect(self.setMode)        
        self.latexButton = QPushButton('LaTex to PDF')
        hbox1.addWidget(self.latexButton)
        self.latexButton.clicked.connect(self.tex2PDF)
        self.latexButton.setEnabled(False)
        
        hbox5 = QHBoxLayout()
        layout.addLayout(hbox5)		
        self.msg = QLabel('msg')
        self.msg.setStyleSheet("background-color: white;")		
        hbox5.addWidget(self.msg)		

        container = QWidget()
        container.setLayout(layout)
        self.setCentralWidget(container)  	# making it central widget
        self.setWindowTitle('Question Bank management: Bulk Data Upload from CSV file')
        
        # Thread creation for upload()
        self.uploadButton.clicked.connect(self.start)	
        self.worker = Worker()
        self.worker_thread = QThread()        
        self.worker.progress.connect(self.update_progress)
        self.worker.completed.connect(self.complete)
        self.work_requested.connect(self.worker.do_work)
        # move worker to the worker thread
        self.worker.moveToThread(self.worker_thread)
        # start the thread
        self.worker_thread.start()    
        
        #self.btn_start = QPushButton('Start', clicked=self.start)
        #hbox1.addWidget(self.btn_start)                    
        
        self.show()

    def start(self):
        self.fileButton.setEnabled(False)
        self.viewButton.setEnabled(False)
        self.openButton.setEnabled(False)
        self.uploadButton.setEnabled(False)
        self.clearButton.setEnabled(False)
        self.dispButton.setEnabled(False)
        self.latexButton.setEnabled(False)
        self.verifyButton.setEnabled(False)
        self.latexCB.setEnabled(False)
        self.work_requested.emit(0)

    def update_progress(self, txt):
        self.msg.setText(txt)
        
    def complete(self,x):
        self.fileButton.setEnabled(True)
        self.viewButton.setEnabled(True)
        self.openButton.setEnabled(True)
        self.uploadButton.setEnabled(True)
        self.clearButton.setEnabled(True)
        self.dispButton.setEnabled(True)
        self.latexButton.setEnabled(True)
        self.verifyButton.setEnabled(True)
        self.latexCB.setEnabled(True)
        
class Worker(QObject):
    progress = Signal(str)
    completed = Signal(int)

    @Slot(int)   
    def do_work(self):
        GUI.upload(self)
        self.completed.emit(0)

app = QApplication(sys.argv)   # Main program
GUI = MainWindow()
app.exec()		# main loop

